/*****************************************************
Chip type               : ATmega8
Program type            : Application
AVR Core Clock frequency: 1,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*****************************************************/

#include <mega8.h>

#include <delay.h>

#define V_bat_ind	3500
#define V_bat_off	3000

#define power_on() 	PORTB.2 = 1
#define power_off() 	PORTB.2 = 0

#define out_on() 	PORTB.0 = 1
#define out_off() 	PORTB.0 = 0

#define point_on() 	PORTD|=0b00001000;
#define point_off() 	PORTD&=0b11110111;

#define button		PINB.1 

unsigned int V_bat = 0;			//   
eeprom unsigned int null = 0;
eeprom unsigned char e_p_level = 1;	//    (1..5) <-> (0.5 .. 2.5) 
unsigned char p_level = 1;
unsigned int p_current = 0;		//   
unsigned int p_cnt = 0;			//    

//     * 1000,   (0, 0.5 .. 2.5)  R = 3.5  (sqrt(P*R))
const unsigned long int ref_array[6]={0,1323000,1871000,2291000,2646000,2958000};	 


unsigned char adc_flag = 0;		//   
unsigned char adc_show_flag = 0;	//   
unsigned int button_cnt = 0;           	//   

unsigned char power_off_flag = 0;	//  

unsigned char button_flag = 0;

unsigned char change_flag = 0;

unsigned int show_cnt = 0;              //  

#define ADC_VREF_TYPE 0xC0

// Read the AD conversion result
unsigned int read_adc(unsigned char adc_input)
        {
        ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
        // Delay needed for the stabilization of the ADC input voltage
        delay_us(10);
        // Start the AD conversion
        ADCSRA|=0x40;
        // Wait for the AD conversion to complete
        while ((ADCSRA & 0x10)==0);
        ADCSRA|=0x10;
        return ADCW;
        }

void V_measure (void)
	{
        DDRC=0x02;
        delay_ms(1);
        
        V_bat = (read_adc(0)+read_adc(0)+read_adc(0)+read_adc(0))*5/4;
        DDRC=0x00;
        }

//    PORTD
void ValueToSegment(unsigned char val)
	{
 	switch(val)
   		{             //fgedhcba
                case 0: PORTD=0b10110111;	break; 
                case 1: PORTD=0b00000110;	break; 
                case 2: PORTD=0b01110011;	break; 
                case 3: PORTD=0b01010111;	break; 
                case 4: PORTD=0b11000110;	break; 
                case 5: PORTD=0b11010101;	break; 
                case 6: PORTD=0b11110101;	break; 
                case 7: PORTD=0b00000111;	break; 
                case 8: PORTD=0b11110111;	break; 
                case 9: PORTD=0b11010111;	break; 
                case 10: PORTD=0b11110100;	break;	//b 
                case 11: PORTD=0b11010110;	break;  //Y
                case 12: PORTD=0b11110001;	break;  //E
                case 13: PORTD=0b11100110;	break;  //H
                case 14: PORTD=0b00000100;	break;  //i
                case 15: PORTD=0b10110000;	break;  //L
                case 16: PORTD=0b01110100;	break;  //o
                case 17: PORTD=0b11100001;	break;  //F
                default: PORTD&=0b00001000; // ,   
   		}
	}
               
// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
	{
	TCNT0=0x83;
	#asm("wdr")
        
        if (++p_cnt == 1000) {p_cnt = 0; adc_flag = 0;}
        if (p_current > p_cnt)
        	{
                out_on();
                point_on();
                if (adc_flag == 0 && adc_show_flag == 0)
                	{
                        V_measure();
                        adc_flag = 1;
                        }
                }
                else
                	{
                        out_off();
                	point_off();
                        }
	
        if (show_cnt) show_cnt--;
        	else 
                	{
                        PORTD&=0b00001000;
                        button_flag = 0;
                        if (change_flag) 
                        	{
                                change_flag = 0;
                                e_p_level = p_level;
                                }
                        }
                
        if (button) button_cnt++;
        	else 
                	{
                        if (button_cnt > 1000 && power_off_flag == 0) {button_cnt = 0; adc_show_flag = 1;}
                        if (button_cnt > 20 && power_off_flag == 0) 
                        	{
                                if (button_flag == 0)
                                	{
                                        button_cnt = 0; 
                                        show_cnt = 5000; 
                                        if (p_level != 6) ValueToSegment(p_level); else ValueToSegment(17); 
                                        button_flag = 1;
                                        }
                                        else
                                        	{
                                                button_cnt = 0; 
                                        	show_cnt = 5000;
                                                if (++p_level == 7) p_level = 1; 
                                        	if (p_level != 6) ValueToSegment(p_level); else ValueToSegment(17);
                                                change_flag = 1; 
                                        	}
                                } 
                        }
	if (button_cnt > 2000) {power_off_flag = 1;}
        }
                       
void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=Out Func1=In Func0=Out 
// State7=T State6=T State5=T State4=T State3=T State2=0 State1=T State0=0 
PORTB=0x00;
DDRB=0x05;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=0 
PORTD=0x00;
DDRD=0xFF;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 125,000 kHz
TCCR0=0x02;
TCNT0=0x83;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer1 Stopped
// Mode: Normal top=0xFFFF
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer2 Stopped
// Mode: Normal top=0xFF
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;

// USART initialization
// USART disabled
UCSRB=0x00;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 7,813 kHz
// ADC Voltage Reference: Int., cap. on AREF
ADMUX=ADC_VREF_TYPE & 0xff;
ADCSRA=0x87;

// SPI initialization
// SPI disabled
SPCR=0x00;

// TWI initialization
// TWI disabled
TWCR=0x00;

// Watchdog Timer initialization
// Watchdog Timer Prescaler: OSC/2048k
#pragma optsize-
WDTCR=0x1F;
WDTCR=0x0F;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Global enable interrupts


delay_ms(2000);
if (button) power_on();

if (null == 0xFFFF)
	{
        null = 0;
        e_p_level = 5;
        }
p_level = e_p_level;

show_cnt = 5000;
ValueToSegment(13);
delay_ms(500);
ValueToSegment(14);
delay_ms(500);
ValueToSegment(p_level);
#asm("sei");

V_measure();

while (1)
      	{
      	if (p_level != 6) p_current = ref_array[p_level]/V_bat;
        	else p_current = 1000;

      	if (V_bat < V_bat_ind)
      		{
                show_cnt = 500;
                ValueToSegment(15);
                delay_ms(1000);
        	}
        
        if (V_bat < V_bat_off || power_off_flag)
      		{
                power_off_flag = 0;
                show_cnt = 1500;
                ValueToSegment(10);
                delay_ms(500);
                ValueToSegment(11);
                delay_ms(500);
                ValueToSegment(12);
                delay_ms(500);
                power_off();
                while (button) delay_ms(10000);
                }
        
        if (adc_show_flag)
        	{
                show_cnt = 1500;
                ValueToSegment(V_bat/1000);
                delay_ms(500);
                ValueToSegment(V_bat%1000/100);
                delay_ms(500);
                ValueToSegment(V_bat%1000%100/10);
                delay_ms(500);
                adc_show_flag = 0;
                }
	}
}
